<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2020 Sword SSL
*  All rights reserved
*
***************************************************************/

/**
 * Service for accessing index models from database for reindexing all the data.
 * /!\ beaucoup de copié/collé depuis tx_mksearch_service_internal_Index
 */
class tx_mksearch_service_internal_Reindex extends tx_mksearch_service_internal_Index
{
    /**
     * @var bool
     */
    private static $indexingInProgress = false;

    /**
     * Database table responsible for reindexing queue management.
     *
     * @var string
     */
    private static $queueTable = 'tx_mksearch_queue_reindex';

    /**
     * @param string $tablename
     * @return mixed
     */
    public function countItemsInQueue($tablename = '')
    {
        $options = [];
        $options['count'] = 1;
        $options['where'] = 'deleted=0 AND being_indexed=0';
        if (strcmp($tablename, '')) {
            $fullQuoted = $this->getDatabaseConnection()->fullQuoteStr($tablename, self::$queueTable);
            $options['where'] .= ($tablename ? ' AND tablename='.$fullQuoted : '');
        }
        $options['enablefieldsoff'] = 1;

        $data = $this->getDatabaseConnection()->doSelect('count(*) As cnt', self::$queueTable, $options);

        return $data[0]['cnt'];
    }

    /**
     * Builds the record for insert.
     *
     * @param string $tableName
     * @param int    $uid
     * @param bool   $prefer
     * @param string $resolver  class name of record resolver
     * @param array  $data
     * @param array  $options
     *
     * @return mixed array: (cr_date,prefer,recid,tablename,data,resolver) | false: if allredy exists
     */
    private function buildRecordForIndex($tableName, $uid, $prefer = false, $resolver = false, $data = false, array $options = [])
    {
        $checkExisting = isset($options['checkExisting']) ? $options['checkExisting'] : true;
        if (false === $resolver) {
            $resolver = tx_mksearch_util_Config::getResolverForDatabaseTable($tableName);
            $resolver = count($resolver) ? $resolver['className'] : '';
        }
        if ($checkExisting) {
            $options = [];
            $options['where'] = 'recid=\''.$uid.'\' AND tablename=\''.$tableName.'\' AND deleted=0 AND being_indexed=0 ';
            $options['enablefieldsoff'] = 1;
            $ret = $this->getDatabaseConnection()->doSelect('uid', self::$queueTable, $options);
            if (count($ret)) {
                return false;
            } // Element déjà dans la file d'attente
        }

        $record = [
            'cr_date' => tx_rnbase_util_Dates::datetime_tstamp2mysql($GLOBALS['EXEC_TIME']),
            'prefer' => (int) $prefer,
            'recid' => $uid,
            'tablename' => $tableName,
            'data' => false !== $data ? (is_array($data) ? serialize($data) : $data) : '',
            'resolver' => !empty($resolver) ? $resolver : '',
        ];

        return $record;
    }

    /**
     * Add a single database record to search index.
     *
     * @param string $tableName
     * @param int    $uid
     * @param bool   $prefer
     * @param string $resolver  class name of record resolver
     * @param array  $data
     * @param array  $options
     *
     * @return bool true if record was successfully spooled
     */
    public function addRecordToIndex($tableName, $uid, $prefer = false, $resolver = false, $data = false, array $options = [])
    {
        if (empty($uid) || empty($tableName)) {
            tx_rnbase_util_Logger::warn(
                'Could not add record to index. No table or uid given.',
                'mksearch',
                [
                    'tablename' => '['.gettype($tableName).'] '.$tableName,
                    'uid' => '['.gettype($uid).'] '.$uid,
                    'trace' => tx_rnbase_util_Debug::getDebugTrail(),
                ]
            );

            return false;
        }

        $record = $this->buildRecordForIndex($tableName, $uid, $prefer, $resolver, $data, $options);

        if (!is_array($record)) {
            return;
        }

        $qid = $this->getDatabaseConnection()->doInsert(self::$queueTable, $record);
        if (tx_rnbase_util_Logger::isDebugEnabled()) {
            tx_rnbase_util_Logger::debug('New record to be reindexed added to queue.', 'mksearch', ['queue-id' => $qid, 'tablename' => $tableName, 'recid' => $uid]);
        }

        return $qid > 0;
    }


    /**
     * Trigger indexing from indexing queue.
     *
     * @param array $config
     *                      no:     Number of items to be indexed
     *                      pid:    trigger only records for this pageid
     *
     * @return array
     */
    public function triggerQueueIndexing($config = [])
    {
        if (!is_array($config)) {
            $config = ['limit' => $config];
        }
        $options = [];
        $options['orderby'] = 'prefer desc, cr_date asc, uid asc';
        $options['limit'] = isset($config['limit']) ? (int) $config['limit'] : 100;
        $options['where'] = 'deleted=0 AND being_indexed = 0';
        $options['enablefieldsoff'] = 1;

        $data = $this->getDatabaseConnection()->doSelect('*', self::$queueTable, $options);

        // Nothing found in queue? Stop
        if (empty($data)) {
            return 0;
        }

        $uids = [];
        $rows = [];
        foreach ($data as $queue) {
            $uids[] = $queue['uid'];
            // daten sammeln
            $rows[$queue['tablename']][] = $queue['recid'];
        }

        $this->getDatabaseConnection()->doUpdate(
            self::$queueTable,
            'uid IN ('.implode(',', $uids).')',
            ['being_indexed' => 1]
        );

        // Trigger update for the found items
        if (!$this->executeQueueData($data, $config)) {
            return [];
        }

        if (1 != $GLOBALS['TYPO3_CONF_VARS']['MKSEARCH_testmode']) {
            $ret = Tx_Rnbase_Database_Connection::getInstance()->doUpdate(
                self::$queueTable,
                'uid IN ('.implode(',', $uids).')',
                ['deleted' => 1, 'being_indexed' => 0]
            );
            $this->deleteOldQueueEntries();
            tx_rnbase_util_Logger::info(
                'Indexing run finished with '.$ret.' items executed.',
                'mksearch',
                ['data' => $data]
            );
        } else {
            tx_rnbase_util_Logger::info(
                'Indexing run finished in test mode. Queue not deleted!',
                'mksearch',
                ['data' => $data]
            );
        }

        return $rows;
    }


    /**
     * Abarbeitung der Indizierungs-Queue.
     *
     * @param array $data   records aus der Tabelle tx_mksearch_queue
     * @param array $config
     *                      pid:    trigger only records for this pageid
     */
    private function executeQueueData($data, array $config = [])
    {
        self::setSignalThatIndexingIsInProgress();

        // alle indexer fragen oder nur von der aktuellen pid?
        if ($config['pid']) {
            $indices = $this->getByPageId($config['pid']);
        } else {
            $indices = $this->findAll();
        }

        tx_rnbase_util_Logger::debug('[INDEXQUEUE] Found '.count($indices).' indices for update', 'mksearch');

        try {
            // Loop through all active indices, collecting all configurations
            foreach ($indices as $index) {
                /* @var $index tx_mksearch_model_internal_Index */
                tx_rnbase_util_Logger::debug('[INDEXQUEUE] Next index is '.$index->getTitle(), 'mksearch');
                // Container for all documents to be indexed / deleted
                $indexDocs = [];
                $searchEngine = tx_mksearch_util_ServiceRegistry::getSearchEngine($index);

                $indexConfig = $index->getIndexerOptions();
                // Ohne indexConfig kann nichts indiziert werden
                if (!$indexConfig) {
                    tx_rnbase_util_Logger::notice(
                        '[INDEXQUEUE] No indexer config found! Re-check your settings in mksearch BE-Module!',
                        'mksearch',
                        ['Index' => $index->getTitle().' ('.$index->getUid().')', 'indexerClass' => get_class($index), 'indexdata' => $data]
                    );
                    continue; // Continue with next index
                }

                if (tx_rnbase_util_Logger::isDebugEnabled()) {
                    tx_rnbase_util_Logger::debug('[INDEXQUEUE] Config for index '.$index->getTitle().' found.', 'mksearch', [$indexConfig]);
                }

                // Parcoure maintenant les enregistrements de données (peut-être aussi une boucle externe ...)
                foreach ($data as $queueRecord) {
                    // Zuerst laden wir den Resolver
                    $resolverClazz = $queueRecord['resolver'] ? $queueRecord['resolver'] : 'tx_mksearch_util_ResolverT3DB';
                    $resolver = tx_rnbase::makeInstance($resolverClazz);
                    try {
                        $dbRecords = $resolver->getRecords($queueRecord);
                        foreach ($dbRecords as $record) {
                            // Diese Records müssen jetzt den jeweiligen Indexern übegeben werden.
                            foreach ($this->getIndexersForTable($index, $queueRecord['tablename']) as $indexer) {
                                //foreach (tx_mksearch_util_Config::getIndexersForTable($queueRecord['tablename']) as $indexer) {
                                if (!$indexer) {
                                    continue;
                                } // Invalid indexer
                                // Collect all index documents
                                list($extKey, $contentType) = $indexer->getContentType();
                                //there can be more than one config for the current indexer
                                //so we execute the indexer with each config that was found.
                                //when one element (tt_content) is indexed by let's say tow indexer configs which
                                //aim to the same index than you should take care that the element
                                //isn't taken by both indexer configs as the doc of the element for
                                //the first config will be overwritten by the second one
                                foreach ($indexConfig[$extKey.'.'][$contentType.'.'] as $aConfigByContentType) {
                                    // config mit der default config mergen, falls vorhanden
                                    if (is_array($indexConfig['default.'][$extKey.'.'][$contentType.'.'])) {
                                        $aConfigByContentType = tx_rnbase_util_Arrays::mergeRecursiveWithOverrule(
                                            $indexConfig['default.'][$extKey.'.'][$contentType.'.'],
                                            $aConfigByContentType
                                        );
                                    }

                                    $aConfigByContentType['queueRecord'] = $queueRecord;

                                    // indizieren!
                                    $doc = $indexer->prepareSearchData(
                                        $queueRecord['tablename'],
                                        $record,
                                        $searchEngine->makeIndexDocInstance($extKey, $contentType),
                                        $aConfigByContentType
                                    );
                                    if ($doc) {
                                        if ($GLOBALS['TCA'][$queueRecord['tablename']]['ctrl']['versioningWS']) {
                                            $this->deleteDocumentIfNotCorrectWorkspace(
                                                $aConfigByContentType,
                                                $record,
                                                $doc
                                            );
                                        }

                                        //add fixed_fields from indexer config if defined
                                        $doc = $this->addFixedFields($doc, $aConfigByContentType);

                                        try {
                                            $indexDocs[$doc->getPrimaryKey(true)] = $doc;
                                        } catch (Exception $e) {
                                            tx_rnbase_util_Logger::warn('[INDEXQUEUE] Invalid document returned from indexer.', 'mksearch', ['Indexer class' => get_class($indexer), 'record' => $record]);
                                        }
                                    }
                                }
                            }
                        }
                    } catch (Exception $e) {
                        debug($e, "service internal index");
                        tx_rnbase_util_Logger::warn(
                            '[INDEXQUEUE] Error processing queue item '.$queueRecord['uid'],
                            'mksearch',
                            [
                                'Exception' => $e->getMessage(),
                                'On' => $e->getFile().'#'.$e->getLine(),
                                'Queue-Item' => $queueRecord,
                                'Trace' => $e->getTraceAsString(),
                            ]
                        );
                    }
                }
                // Finally, actually do the index update, if there is sth. to do:
                if (count($indexDocs)) {
                    $elasticIndexesOpenCloseEnbled = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance(\TYPO3\CMS\Core\Configuration\ExtensionConfiguration::class)
                        ->get('mksearch')['elasticIndexesOpenCloseEnbled'];
                    if($elasticIndexesOpenCloseEnbled){$searchEngine->openIndex($index, true);}
                    foreach ($indexDocs as $doc) {
                        try {
                            //debug($doc, 'document');
                            if ($doc->getDeleted()) {
                                $key = $doc->getPrimaryKey();
                                $searchEngine->indexDeleteByContentUid($key['uid']->getValue(), $key['extKey']->getValue(), $key['contentType']->getValue());
                            } else {
                                $searchEngine->indexUpdate($doc);
                            }
                        } catch (Exception $e) {
                            debug($e, 'internal index exception');
                            tx_rnbase_util_Logger::fatal(
                                '[INDEXQUEUE] Fatal error processing search document!',
                                'mksearch',
                                [
                                    'Exception' => $e->getMessage(),
                                    'document' => $doc->__toString(),
                                    // getData liefert die IndexerField Objekte.
                                    // Diese wandeln wir in Strings um, da sonst die Objekte
                                    // nicht wiederhergesetllt werden können und Serialisiert auch zu viel Speicher rauben!
                                    'data' => array_map('strval', $doc->getData()),
                                ]
                            );
                        }
                    }
                    $searchEngine->commitIndex();
                    //shall something be done after indexing?
                    $searchEngine->postProcessIndexing($index);
                    //that's it
                    $elasticIndexesOpenCloseEnbled = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance(\TYPO3\CMS\Core\Configuration\ExtensionConfiguration::class)
                        ->get('mksearch')['elasticIndexesOpenCloseEnbled'];
                    if($elasticIndexesOpenCloseEnbled){$searchEngine->closeIndex();}
                }
            }
        } catch (Exception $e) {
            tx_rnbase_util_Logger::fatal(
                '[INDEXQUEUE] Fatal error processing queue occured! The queue is left as it is so the indexing can be tried again.',
                'mksearch',
                [
                    'Exception' => $e->getMessage(),
                    'Queue-Items' => $data,
                ]
            );
            self::removeSignalThatIndexingIsInProgress();

            return false;
        }

        self::removeSignalThatIndexingIsInProgress();

        return true;
    }

    private static function setSignalThatIndexingIsInProgress()
    {
        self::$indexingInProgress = true;
    }

    private static function removeSignalThatIndexingIsInProgress()
    {
        self::$indexingInProgress = false;
    }

    /**
     * Returns all indexers for an index and a specific tablename.
     *
     * @param tx_mksearch_model_internal_Index $index
     * @param string $tablename
     * @return array
     */
    private function getIndexersForTable($index, $tablename)
    {
        $ret = [];
        $indexers = tx_mksearch_util_Config::getIndexersForTable($tablename);
        foreach ($indexers as $indexer) {
            if ($this->isIndexerDefined($index, $indexer)) {
                $ret[] = $indexer;
            }
        }

        return $ret;
    }

}

if (defined('TYPO3_MODE') && $GLOBALS['TYPO3_CONF_VARS'][TYPO3_MODE]['XCLASS']['ext/mksearch/service/internal/class.tx_mksearch_service_internal_Reindex.php']) {
    include_once $GLOBALS['TYPO3_CONF_VARS'][TYPO3_MODE]['XCLASS']['ext/mksearch/service/internal/class.tx_mksearch_service_internal_Reindex.php'];
}
